------------------------------------------------------------------------
--     Main.vhd -- Simple stopwatch demo for Nexys3
------------------------------------------------------------------------
-- Author:  Cong He
--          Copyright 2012 Digilent, Inc.
------------------------------------------------------------------------
-- This is the top module of stopwatch demo on Nexys3.
-- It was developed using the Xilinx WebPack tools.
--
--  Inputs:
--		clk		- system clock 100MHz
--    swt      - slide switch for timer enable 
--		btn		   - buttons on the Nexys3 board
--
--  Outputs:
--		an		   - anode lines for the 7-seg displays on Nexys3
--		seg		- cathodes (segment lines) for the displays on Nexys3
--
------------------------------------------------------------------------
-- Revision History:
--  3/30/2012(HC): Created
------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity main is
    Port (clk	: in STD_LOGIC;
			 swt  : in STD_LOGIC;
          btn  : in STD_LOGIC_VECTOR(3 downto 0);
          an   : out STD_LOGIC_VECTOR(5 downto 0);
          seg  : out STD_LOGIC_VECTOR(7 downto 0));
end main;

architecture Behavioral of main is

	------------------------------------------------------------------------
	-- Component Declarations
	------------------------------------------------------------------------
	
	component clock_divider
		Port (rst,clk : in  STD_LOGIC;
            cclk : out  STD_LOGIC);
	end component;
		
	component state_machine
		Port (rst,cclk : in STD_LOGIC;
			   startBtn,stopBtn,incBtn : in  STD_LOGIC;
			   timerEN : out STD_LOGIC);
	end component;
		
	component counter
		Port ( rst, cclk : in  STD_LOGIC;
			    timerEN : in STD_LOGIC;
             cntr : out  STD_LOGIC_VECTOR (23 downto 0));
	end component;
		
	component seven_seg_controller
		Port ( rst, cclk : in STD_LOGIC;
			    dispEN : in STD_LOGIC;
				 dpSel : in STD_LOGIC_VECTOR(2 downto 0);
			    cntr : in STD_LOGIC_VECTOR(23 downto 0);
			    seg : out STD_LOGIC_VECTOR(7 downto 0);
			    an : out STD_LOGIC_VECTOR(5 downto 0));
	end component;
		
	------------------------------------------------------------------------
	-- General control and timing signals
	------------------------------------------------------------------------		

	signal cclk, rst, startBtn, stopBtn, incBtn, timerEN, dispEN : STD_LOGIC;
	
	------------------------------------------------------------------------
	-- Data path signals
	------------------------------------------------------------------------
	signal cntr : STD_LOGIC_VECTOR(23 downto 0);
	signal segdat : STD_LOGIC_VECTOR(3 downto 0);
	
	signal seven_seg_d : STD_LOGIC_VECTOR(7 downto 0);
	
	signal an_d: STD_LOGIC_VECTOR(5 downto 0);
	
	------------------------------------------------------------------------
	-- Implementation
	------------------------------------------------------------------------

	begin
	
	Inst_clock_divider: clock_divider PORT MAP(
		rst => rst,
		clk => clk,
		cclk => cclk
	);
	
	Inst_state_machine: state_machine PORT MAP(
		rst => rst,
		cclk => cclk,
		startBtn => startBtn,
		stopBtn => stopBtn,
		incBtn => incBtn,
		timerEN => timerEN
	);	
	
	Inst_counter: counter PORT MAP(
		rst => rst,
		cclk => cclk,
		timerEN => timerEN,
		cntr => cntr
	);
	
	Inst_seven_seg_controller: seven_seg_controller PORT MAP(
		rst => rst,
		cclk => cclk,
		dispEN => dispEN,
		dpSel => "111",
		cntr => cntr,
		seg => seven_seg_d,
		an => an_d
	);	
	
	rst <= btn(0); -- button0 to issue a reset signal
	startBtn <= btn(1); -- start counterting 
	stopBtn <= btn(2); -- stop counterting
	incBtn <= btn(3); -- counter number increases 1 after a single press
	dispEN <= swt; -- 7-seg display enable
	
	seg <= not seven_seg_d;
	an <= not an_d;
	
end Behavioral;