------------------------------------------------------------------------
--	sync_gen.vhd  -- Synchronize Signal Generator
------------------------------------------------------------------------
-- Author: He Cong
--	Copyright 2011 Digilent, Inc.
------------------------------------------------------------------------
-- Module description
--   The module generates synchronize signals for 640x480 VGA @ 60Hz
--
--  Inputs:
--		PIX_CLK_I - 75MHz Pixel Clock Input
--
--  Outputs:
--	   HSYNC_O - horizontal synchronize signal
--	   VSYNC_O - vertical synchronize signal
--	   PIX_CNT_O - counter for pixels in a line
--	   LINE_CNT_O - conter for lines in a frame
--
------------------------------------------------------------------------
-- Revision History:
--
--	13Apr2011(CongH): created
--	13Apr2011(SteveW): coding sytle
--
------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity sync_gen is
	Port( 
		PIX_CLK_I		: in  std_logic;
		HSYNC_O			: out std_logic;
		VSYNC_O			: out std_logic;
		DE_O				: out std_logic;
		PIX_CNT_O		: out std_logic_vector (10 downto 0);
		LINE_CNT_O		: out std_logic_vector (9 downto 0)			 
	);
end sync_gen;

architecture Behavioral of sync_gen is

	------------------------------------------------------------------------
	-- Component Declarations
	------------------------------------------------------------------------

	------------------------------------------------------------------------
	-- Signals Declarations
	------------------------------------------------------------------------

   -- Sync signals
	signal de_i : std_logic := '0'; -- internal de signal
	
	signal line_tick_cnt  : std_logic_vector (9 downto 0) := "0000000000";  --10-bit counter for lines
   signal pix_tick_cnt : std_logic_vector (10 downto 0):= "00000000000"; --11-bit counter for pixels
	
	signal hsync_i : std_logic := '0'; -- Internal vsync,so we can check its state
	signal vsync_i : std_logic := '0'; -- Internal hsync," " "
	
	--Sync compare constants
--	constant cTickLineChk  : std_logic_vector (9 downto 0)  := "1011101110";    --pixel H(750)
--	constant cTickPixelChk : std_logic_vector (10 downto 0) := "11001110010";   --pixel W(1650)
--	constant cTickVsyncPulseChk : std_logic_vector (2 downto 0) := "101";       --vsync pulse width(line clocks) 5
--	constant cTickHsyncPulseChk : std_logic_vector (5 downto 0) := "101000";    --hsync pulse width(pixel clocks)40
--	
--	constant displeftX   : STD_LOGIC_VECTOR (10 DOWNTO 0) := "00100000100";   -- 260  px 
--	constant disprightX  : STD_LOGIC_VECTOR (10 DOWNTO 0) := "11000000100";   -- 1540 px 
--	constant dispupY     : STD_LOGIC_VECTOR (9 DOWNTO 0)  := "0000011001";    -- 25
--	constant dispdownY   : STD_LOGIC_VECTOR (9 DOWNTO 0)  := "1011101001";    -- 745
	constant cTickLineChk  : INTEGER := 750;    --pixel H(750)
	constant cTickPixelChk : INTEGER := 1650;   --pixel W(1650)
	constant cTickVsyncPulseChk : INTEGER := 5;       --vsync pulse width(line clocks) 5
	constant cTickHsyncPulseChk : INTEGER := 40;    --hsync pulse width(pixel clocks)40
	
	constant displeftX   : INTEGER := 260;   -- 260  px 
	constant disprightX  : INTEGER := 1540;   -- 1540 px 
	constant dispupY     : INTEGER := 25;    -- 25
	constant dispdownY   : INTEGER := 745;    -- 745

------------------------------------------------------------------------
-- Implementation
------------------------------------------------------------------------
begin

	-- Tick Counter
   process(PIX_CLK_I)
	begin
	   if PIX_CLK_I'Event and PIX_CLK_I = '1' then
			if(pix_tick_cnt = cTickPixelChk - 1) then
				pix_tick_cnt <= (others => '0');
				if (line_tick_cnt = cTickLineChk - 1) then
					line_tick_cnt <= (others => '0');
				else
					line_tick_cnt <= line_tick_cnt + 1;
				end if;
			else
				pix_tick_cnt <= pix_tick_cnt + 1;
			end if;
		end if;
	end process;
	
	vsync_i <= '1' when (line_tick_cnt < cTickVsyncPulseChk) else
				  '0';
	hsync_i <= '1' when (pix_tick_cnt < cTickHsyncPulseChk) else
				  '0';
	de_i <= 	'1' when ((pix_tick_cnt >= displeftX) and (pix_tick_cnt < disprightX)
					and (line_tick_cnt >= dispupY) and (line_tick_cnt < dispdownY)) else
				'0';
			
	data_enable : process(PIX_CLK_I)
	begin
	   if PIX_CLK_I'Event and PIX_CLK_I = '1' then
		   DE_O <= de_i;
			HSYNC_O <= hsync_i;
			VSYNC_O <= vsync_i;
			PIX_CNT_O <= pix_tick_cnt;
			LINE_CNT_O <= line_tick_cnt;
		end if;
	end process;

end Behavioral;

