/* ------------------------------------------------------------------ */
/* hdmi_encoder.v  -- HDMI Encoder with XSVI interface					 */
/* ------------------------------------------------------------------ */
/* Author: WANG, Tinghui (Steve)													 */
/*	Copyright 2001 Digilent, Inc.													 */
/* ------------------------------------------------------------------ */
/* Module description																 */
/*		HDMI Encoder module with XSVI interface								 */
/*																							 */
/*  Inputs:																				 */
/*		pix_clk - pixel clock														 */
/* 	pix_clkx2 - pixel clock (twice the frequency)						 */
/* 	pix_clkx10 - pixel clock (10 times the frequency)					 */
/* 	locked - clock stable signal												 */
/* 	rstin - reset (high effective)											 */
/* 	video_din - video pixel data												 */
/* 	hsync - horizontal synchronize signal									 */
/* 	vsync - vertical synchronize signal										 */
/* 	de - video data enable signal												 */
/* 	aux0_din - audio data 0														 */
/* 	aux1_din - audio data 1														 */
/* 	aux2_din - audio data 2														 */
/* 	ade - audio enable signal													 */
/*																							 */
/*  Outputs:																			 */
/*		TMDS_TX - hdmi output differential signal pair						 */
/*		TMDSB_TX	- hdmi output differential signal pair						 */
/*																							 */
/* ------------------------------------------------------------------ */
/* Revision History:																	 */
/*																							 */
/*	01Feb2011 (SteveW): modified													 */
/*																							 */
/* ------------------------------------------------------------------ */

//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2007                 www.xilinx.com
//
//  XAPP460
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       hdmi_encoder.v
//
//  Description :     hdmi_encoder 
//
//  Date - revision : March 2008 - 1.0.0
//
//  Author :          Bob Feng
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors makeand you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specificallydisclaims any
//              implied warranties of merchantability, non-infringement,or
//              fitness for a particular purpose. Xilinx does notwarrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designswill be
//              uninterrupted or error free, or that defects in theDesigns
//              will be corrected. Furthermore, Xilinx does not warrantor
//              make any representations regarding use or the results ofthe
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits,cost
//              or procurement of substitute goods or services, or forany
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on anytheory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure ofthe
//              essential purpose of any limited remedies herein.
//
//  Copyright  2004 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
`timescale 1 ps / 1ps

module hdmi_encoder # (
  parameter SDATAINVERT = "FALSE", //Invert SDATA before serilize it
  parameter C_DATA_WIDTH          = 8,
  parameter C_NUM_DATA_CHANNELS   = 3,  
  parameter C_DDR_OUT             = 0  
)(
// clock and control signals
input         pix_clk,
input         pix_clkx2,
input         pix_clkx10,
input         LOCKED,
input         rstin,            // reset
// fit to XSVI standard port
input [C_DATA_WIDTH*C_NUM_DATA_CHANNELS -1 : 0] video_din, // video data, RGB888
input         hsync,            // hsync data
input         vsync,            // vsync data
input         vde,              // video data enable
// audio part
input [3:0]   aux0_din,
input [3:0]   aux1_din,
input [3:0]   aux2_din,
input         ade,              // audio/aux data enable
// hdmi output ports
output	[3:0] TMDS_TX,
output  [3:0] TMDSB_TX	    	// data outputs to ddr in IOB, bit 3(and 7) is clock
);

wire	[7:0]   blue_din;         // Blue data in
wire	[7:0]   green_din;        // Green data in
wire	[7:0]   red_din;          // Red data in

assign blue_din = {video_din[7:0]};
assign green_din = {video_din[15:8]};
assign red_din = {video_din[23:16]};

wire 	[9:0]   red;
wire 	[9:0]   green;
wire 	[9:0]   blue;
wire 	[29:0]  s_data ;

reg  ctl0, ctl1, ctl2, ctl3;

parameter DILNDPREAM = 4'b1010;
parameter VIDEOPREAM = 4'b1000;
parameter NULLCONTRL = 4'b0000;

always @ (posedge pix_clk) begin
  if(vde)
    {ctl0, ctl1, ctl2, ctl3} <=#1 VIDEOPREAM;
  else if(ade)
    {ctl0, ctl1, ctl2, ctl3} <=#1 DILNDPREAM;
  else
    {ctl0, ctl1, ctl2, ctl3} <=#1 NULLCONTRL;
end

wire [7:0] blue_dly, green_dly, red_dly;
wire [3:0] aux0_dly, aux1_dly, aux2_dly;
wire       hsync_dly, vsync_dly, vde_dly, ade_dly;

srldelay # (
  .WIDTH(40),
  .TAPS(4'b1010)
) srldly_0 (
  .data_i({blue_din, green_din, red_din, aux0_din, aux1_din, aux2_din, hsync, vsync, vde, ade}),
  .data_o({blue_dly, green_dly, red_dly, aux0_dly, aux1_dly, aux2_dly, hsync_dly, vsync_dly, vde_dly, ade_dly}),
  .clk(pix_clk)
);

encode # (
  .CHANNEL("BLUE")
)encb (
	.clkin	(pix_clk),
	.rstin	(rstin),
	.vdin		(blue_dly),
  .adin   (aux0_dly),
	.c0     (hsync_dly),
	.c1     (vsync_dly),
	.vde		(vde_dly),
	.ade		(ade_dly),
	.dout		(blue)) ;

encode # (
  .CHANNEL("GREEN")
) encg (
	.clkin	(pix_clk),
	.rstin	(rstin),
	.vdin		(green_dly),
  .adin   (aux1_dly),
	.c0     (ctl0),
	.c1     (ctl1),
	.vde		(vde_dly),
	.ade		(ade_dly),
	.dout		(green)) ;

encode # (
  .CHANNEL("RED")
) encr (
	.clkin	(pix_clk),
	.rstin	(rstin),
	.vdin		(red_dly),
  .adin   (aux2_dly),
	.c0     (ctl2),
	.c1     (ctl3),
	.vde		(vde_dly),
	.ade		(ade_dly),
	.dout		(red)) ;

/*assign s_data = {red[9], green[9], blue[9], red[8], green[8], blue[8],
	   	 red[7], green[7], blue[7], red[6], green[6], blue[6],
	   	 red[5], green[5], blue[5], red[4], green[4], blue[4],
	   	 red[3], green[3], blue[3], red[2], green[2], blue[2],
	   	 red[1], green[1], blue[1], red[0], green[0], blue[0]} ;
*/
reg toggle;
always@(posedge pix_clkx2 or posedge rstin)
    if(rstin )
      toggle <= 0;
    else
      toggle <= ~toggle;

reg [4:0] tmdsclkint;
always@(posedge pix_clkx2)
		if(toggle)
			tmdsclkint <= 5'b11111;
		else
			tmdsclkint <= 5'b00000;

wire clkfb;

wire tx_clkx10;
wire tx_serdesstrobe;
wire LOCK;

BUFPLL #(
    .DIVIDE(5)
)
tx_bufpll_inst(
    .IOCLK(tx_clkx10),
    .LOCKED(LOCKED),
    .LOCK(LOCK),
    .PLLIN(pix_clkx10),
    .GCLK(pix_clkx2),
    .SERDESSTROBE(tx_serdesstrobe));

assign s_data = {red[9:5], green[9:5], blue[9:5],
                 red[4:0], green[4:0], blue[4:0]};
reg [29:0] s_data_q;

always @ (posedge pix_clk) begin
  if(SDATAINVERT == "TRUE")
    s_data_q <= ~s_data;
  else
    s_data_q <= s_data;
end
wire [4:0] tmds_blue;
wire [4:0] tmds_green;
wire [4:0] tmds_red;
convert_30to15_fifo serialise (
	.clk			(pix_clk),
	.clkx2		(pix_clkx2),
	.datain		(s_data_q),
	.rst			(rstin),
	.dataout	({tmds_red,tmds_green,tmds_blue})) ;

wire [2:0] TMDSINT;
wire    tmdsclk;
serdes_n_to_1 serial_b(
    .ioclk(tx_clkx10),
    .serdesstrobe(tx_serdesstrobe),
    .reset(rstin),
    .gclk(pix_clkx2),
    .datain(tmds_blue),
    .iob_data_out(TMDSINT[0]));

serdes_n_to_1 serial_g(
    .ioclk(tx_clkx10),
    .serdesstrobe(tx_serdesstrobe),
    .reset(rstin),
    .gclk(pix_clkx2),
    .datain(tmds_green),
    .iob_data_out(TMDSINT[1]));

serdes_n_to_1 serial_r(
    .ioclk(tx_clkx10),
    .serdesstrobe(tx_serdesstrobe),
    .reset(rstin),
    .gclk(pix_clkx2),
    .datain(tmds_red),
    .iob_data_out(TMDSINT[2]));

serdes_n_to_1 serial_clk(
    .ioclk(tx_clkx10),
    .serdesstrobe(tx_serdesstrobe),
    .reset(rstin),
    .gclk(pix_clkx2),
    .datain(tmdsclkint),
    .iob_data_out(tmdsclk));


OBUFDS #(
    .IOSTANDARD("DEFAULT")
)
OBUFDS_B(
    .O(TMDS_TX[0]),
    .OB(TMDSB_TX[0]),
    .I(TMDSINT[0]));

OBUFDS #(
    .IOSTANDARD("DEFAULT")
)
OBUFDS_G(
    .O(TMDS_TX[1]),
    .OB(TMDSB_TX[1]),
    .I(TMDSINT[1]));
OBUFDS #(
    .IOSTANDARD("DEFAULT")
)
OBUFDS_R(
    .O(TMDS_TX[2]),
    .OB(TMDSB_TX[2]),
    .I(TMDSINT[2]));

OBUFDS #(
    .IOSTANDARD("DEFAULT")
)
OBUFDS_CLK(
    .O(TMDS_TX[3]),
    .OB(TMDSB_TX[3]),
    .I(tmdsclk));

endmodule
